@@ -1,3 +1,5 @@
+use std::borrow::Cow;
+
 use rocket::form::Form;
 use rocket::http::impl_from_uri_param_identity;
 use rocket::http::uri::fmt::Formatter;
@@ -13,6 +15,7 @@ use rocket::{Build, Rocket};
 
 use crate::error::MyError;
 use crate::repository;
+use crate::repository::RepositoryCapability;
 use crate::templates;
 use crate::templates::render_search_results;
 use crate::templates::{
@@ -61,18 +64,24 @@ impl<'r> WikiPagePath<'r> {
         }
     }
 
-    fn file_name_and_extension(&self) -> Option<(&str, &str)> {
+    fn file_name(&self) -> Option<&str> {
+        let (file_name, _) = self.segments.split_last()?;
+        Some(file_name)
+    }
+
+    fn file_stem_and_extension(&self) -> Option<(&str, &str)> {
         let (file_name, _) = self.segments.split_last()?;
         file_name.rsplit_once('.')
     }
 
-    fn file_name(&self) -> Option<&str> {
-        Some(self.file_name_and_extension()?.0)
+    #[cfg(test)]
+    fn file_stem(&self) -> Option<&str> {
+        Some(self.file_stem_and_extension()?.0)
     }
 
     #[cfg(test)]
     fn file_extension(&self) -> Option<&str> {
-        Some(self.file_name_and_extension()?.1)
+        Some(self.file_stem_and_extension()?.1)
     }
 
     fn breadcrumbs_helper<F: Fn(&'r [&'r str]) -> Origin>(
@@ -152,6 +161,22 @@ impl<'r, 'o: 'r> Responder<'r, 'o> for MyError {
 #[derive(FromForm)]
 struct PageEditForm<'r> {
     content: &'r str,
+    message: &'r str,
+}
+
+fn edit_save_inner(
+    path: WikiPagePath,
+    content: Form<PageEditForm<'_>>,
+    w: Wiki,
+) -> Result<response::Redirect, MyError> {
+    let message = if content.message.trim().is_empty() {
+        let message = default_edit_message(&path);
+        Cow::Owned(message)
+    } else {
+        Cow::Borrowed(content.message.trim())
+    };
+    w.write_file(&path.segments, &message, content.content)?;
+    Ok(response::Redirect::to(uri!(page(path))))
 }
 
 #[post("/edit/<path..>", data = "<content>")]
@@ -160,27 +185,46 @@ fn edit_save(
     content: Form<PageEditForm<'_>>,
     w: Wiki,
 ) -> Result<response::Redirect, MyError> {
-    w.write_file(&path.segments, content.content)?;
-    Ok(response::Redirect::to(uri!(page(path))))
+    edit_save_inner(path, content, w)
 }
 
-#[get("/edit/<path..>")]
-fn edit_view(path: WikiPagePath, w: Wiki) -> Result<response::content::Html<String>, MyError> {
+fn default_edit_message(path: &WikiPagePath) -> String {
+    format!("Update {}", path.file_name().expect("Ill-formed path"))
+}
+
+fn edit_view_inner(
+    path: WikiPagePath,
+    w: Wiki,
+) -> Result<response::content::Html<String>, MyError> {
     let content = w.read_file(&path.segments).unwrap_or_else(|_| vec![]);
     let content = std::str::from_utf8(&content)?;
     let post_url = uri!(edit_save(&path));
     let view_url = uri!(page(&path));
-    let file_stem = path.file_name().expect("Ill-formed path");
+    let title = format!("Editing {}", path.file_name().expect("Ill-formed path"));
+    let message_placeholder = if w
+        .repo_capabilities()
+        .contains(RepositoryCapability::SUPPORTS_EDIT_MESSAGE)
+    {
+        Some(default_edit_message(&path))
+    } else {
+        None
+    };
     let html = render_edit_page(
-        file_stem,
+        &title,
         &post_url.to_string(),
         &view_url.to_string(),
+        message_placeholder,
         content,
         path.page_breadcrumbs(),
     )?;
     Ok(response::content::Html(html))
 }
 
+#[get("/edit/<path..>")]
+fn edit_view(path: WikiPagePath, w: Wiki) -> Result<response::content::Html<String>, MyError> {
+    edit_view_inner(path, w)
+}
+
 fn page_response(
     page: crate::page::Page,
     path: &WikiPagePath,
@@ -194,7 +238,7 @@ fn page_response(
 fn page_inner(path: WikiPagePath, w: Wiki) -> Result<WikiPageResponder, MyError> {
     match w.read_file(&path.segments) {
         Ok(bytes) => {
-            let file_info = path.file_name_and_extension();
+            let file_info = path.file_stem_and_extension();
             Ok(match file_info {
                 Some((file_stem, file_ext)) => {
                     match crate::page::get_page(file_stem, file_ext, &bytes, w.settings())? {
@@ -220,7 +264,7 @@ fn page_inner(path: WikiPagePath, w: Wiki) -> Result<WikiPageResponder, MyError>
                     page(path)
                 ))))
             } else {
-                match path.file_name_and_extension() {
+                match path.file_stem_and_extension() {
                     Some((file_stem, "md")) => {
                         let create_url = uri!(edit_view(&path));
                         Ok(WikiPageResponder::PagePlaceholder(
@@ -340,7 +384,7 @@ mod tests {
         let parsed = WikiPagePath::from_slice(input);
         assert_eq!(
             Some(expected_file_stem),
-            parsed.file_name(),
+            parsed.file_stem(),
             "Unexpected file_stem while parsing request: {:?}",
             input
         );
@@ -374,11 +418,11 @@ mod tests {
     fn test_request_path_parse_unsupported() {
         let empty = WikiPagePath::new(vec![]);
         assert!(empty.directories().is_empty());
-        assert!(empty.file_name_and_extension().is_none());
+        assert!(empty.file_stem_and_extension().is_none());
 
         let extensionless_file = WikiPagePath::new(vec!["README"]);
         assert!(extensionless_file.directories().is_empty());
-        assert!(extensionless_file.file_name_and_extension().is_none());
+        assert!(extensionless_file.file_stem_and_extension().is_none());
     }
 
     #[test]
